function LarvMort_st = plankton_mort(Sp_name,St_name)

% Function to calculate larval mortality estimates based on an updated
% version of Aksnes & Ohman's (1996 L&O) vertical life table approach.

% Author: Will White (whitejw@uncw.edu)
% This code accompanies:
% White JW, Morgan SG, Fisher JL. Improved estimates of
% planktonic larval mortality rates are lower than expected.
% Ecology
% Please cite accordingly.

% version: April 2014

%%-----------------------------------------------------------------------
% Input arguments: 

% Sp - string giving name of species to be analyzed (must be same as one of
% the names in Larv_data.mat

% St - string giving name of stage pair to be analyzed for the species in
% Sp.
% As currently formatted, must be one of:
%   EarlyMid
%   MidLate
%   LatePL
% (and must be a stage that was collected for the species in question)
%%-----------------------------------------------------------------------

%%-----------------------------------------------------------------------
% Output arguments:

% LarvMort_st - Structure file containing a partition for each species
% and stage analyzed.  Each partition has a sub-branch for each MCMC chain,
% which has the following fields:
%       theta_mc % mean of posterior distribution for Theta (mortality
%       rate)
%       loglik % mean of the posterior probability distribution
%       p(parameters|data)
%       x_dist: % MCMC chain containing the posterior parameter distribution.  1st column is negbin
%       parameter k, 2nd column is mortality rate parameter Theta
%       L_dist: % MCMC chain containing the posterior probability
%       distribution.
%%-----------------------------------------------------------------------

%%-----------------------------------------------------------------------
% Data requirements:

% This code is designed to operate with the data presented in the White et
% al. manuscript.  Some modification may be required to operate on datasets
% with different sampling frequencies or levels of replication.

% Required files:  larv_data.mat

% This file contains:
%   larv - structure file with .data and .textdata partitions. This file
%   is the result of performing csvread.m on the Larval_abundance_data.csv
%   data file.  See that file and metadata for information on its
%   formatting.
%
%   larv_dur - structure file containing stage durations for each species.
%   See the example provided for proper formatting
%
%   Sp - cell array containing strings with all of the species names in
%   larv
%
%   St - cell array containing the list of potential stage names
%
%   Days - vector of the numerical sampling days (1-63) in the dataset.
%
%   See the examples provided for proper formatting
%%-----------------------------------------------------------------------

%%-----------------------------------------------------------------------
% Parameters that may be modified by the user:

% MCMC parameters
  N = 1e4; % chain length
  NC = 3; % multiple chains
  Nseeds = 5; % need to average across several different simulations
             
% Datename (for appending to file saving results)
Datename = '20Apr2013';
  
% Initial parameter values
theta_init = -1; % on a log10 scale
k_init = 1;


% Priors:
% Theta (normal)
theta_pm = -1; 
theta_ps = 1e4;
% K (lognormal)
k_pm = 0;
k_ps = 1e4;

% CV multipliers for delayed rejection
% (at each step of delayed rejection the coefficient of variation (CV) for the
% parameter in question steps down; this is the series of CV values to be
% used
                cv_fact = repmat(1./([0.5 1 5:1:10])',[1,2]); 

% constraints on estimation (log10 scale)
                min_con = [-2 -10];
                max_con = [2, 2];
%%-----------------------------------------------------------------------
                

%%-----------------------------------------------------------------------
% Load in data
load Larv_data.mat larv larv_dur Days St Sp

% Here are the SpeciesNames used in the original analysis:
%SpeciesNames = {'Balanuscrenatus','Balanusglandula','Chthamalusdalli','Pagurid',...
%                'Pinnotheridae','Porcellanid','Hemioregonensis'};
           
SpeciesNames = {Sp_name};
%%-----------------------------------------------------------------------


% Establish a random stream for use with the candidate-generating function
% and the Meteropolis step
stream_null = RandStream('mt19937ar','seed',sum(clock));

% Pre-allocate some variables
LarvMort_st = struct([]);
for s = 1:length(SpeciesNames)
    sp = SpeciesNames{s};
    for nc = 1:NC
                LarvMort_st(1).(sp).chain(nc).k_mc = [];
                LarvMort_st(1).(sp).chain(nc).theta_mc = [];
                LarvMort_st(1).(sp).chain(nc).loglik = [];
                LarvMort_st(1).(sp).chain(nc).x_dist = [];
                LarvMort_st(1).(sp).chain(nc).L_dist = [];
    end
end % end loop over species


% Setup the data, extract stage durations
for s =  1:length(SpeciesNames) 
    
    sp = SpeciesNames{s};
    StageNames = larv_dur.(sp).StageNames;
   
    % Implement a fix to deal with zero observations, etc
    switch sp
        case {'Porcellanid','Pinnotheridae','Fabiasub'}
            StageNames = StageNames(~strcmp(StageNames,'PL'));
        case {'Pagurid'}
            StageNames = StageNames(~strcmp(StageNames,'PL'));
            StageNames = StageNames(~strcmp(StageNames,'late'));
        case {'Chthamalusdalli'}
            StageNames = StageNames(~strcmp(StageNames,'PL')); 
            StageNames = StageNames(~strcmp(StageNames,'early')); 
        case {'Balanuscrenatus', 'Balanusglandula','Balanusnubilus'} 
            StageNames = StageNames(~strcmp(StageNames,'early')); % no early larval observations
    end
    
    % Switch to allow stage-specific estimations
    switch Sp_name
        case {'Balanuscrenatus','Balanusglandula'}
            switch St_name
                case 'MidLate'
                    StageNames = {'mid','late'};
                case 'LatePL'
                    StageNames = {'late','PL'};
            end
         
        case {'Pinnotheridae'}
            switch St_name
                case 'EarlyMid'
                    StageNames = {'early' 'mid'};
                case 'MidLate'
                    StageNames = {'mid','late'};
            end
        
    end %end switch Sp_name
    
    SN = length(StageNames);
    r_data = cell(SN-1,1);
    a0_d = nan([1,SN-1]);
    a1_d = nan([1,SN-1]);
    a0_t = nan([1,length(Days)]);
    a1_t = a0_t;
    
                % setup the dataset for each stage
                for sn = 1:SN-1
              
                a0 = larv.data(strcmp(sp,Sp) & strcmp(StageNames{sn},St),:); % density of stage i of Species s on all days
                a1 = larv.data(strcmp(sp,Sp) & strcmp(StageNames{sn+1},St),:);   % density of stage i+1 of Species s on all days
                
                a0_d(sn) = larv_dur.(sp).Duration(sn);
                a1_d(sn) = larv_dur.(sp).Duration(sn+1);
                
                for t = 1:length(Days)
                    a0_t(t) = mean(a0(a0(:,1)==Days(t),3));
                    a1_t(t) = mean(a1(a1(:,1)==Days(t),3));
                end  % end for days
                
                % Implement limits to temporal range of data
                switch sp
                    case {'Balanusnubilus'}
                        t_OK = Days >= 17 & Days <= 35;
                    case {'Chthamalusdalli'}
                        t_OK = Days >= 17 & Days <= 59;
                    case {'Porcellanid'}
                        t_OK = Days <= 59;
                    otherwise
                        t_OK = true(size(Days));
                end
                
                a0_t = a0_t(t_OK); 
                a1_t = a1_t(t_OK); 
                
                r = a0_t./a1_t;
                r = r(~isnan(r) & ~isinf(r));
                r_data{sn} = r;
                end % end loop over SN
                
                
                %%%%%%%%%%%%%%%%%%%%%%%
                % Implement MCMC
                
                   
                stepmax = size(cv_fact,1); % Max number of steps       
                CV = [cv_fact(:,1), repmat(cv_fact(:,2),[1,length(theta_init)])];
               
                seed = 1001:1:(Nseeds*NC+1000); % random seeds
                seed = reshape(seed,[NC,Nseeds]);
                
                for nc = 1:NC % run multiple chains
                
                p = 1; % variable index
                
                x = nan([N,length(theta_init)+1]);
                L = ones(N,1)*NaN;
                
                % initialize:
                x(1,:) = [k_init theta_init];
                % need to vary initial conditions
                x(1,:) = randn(stream_null,1,size(x,2)).*CV(1,:).*x(1,:) + x(1,:);
                % impose constraints
                x(1,:) = max(x(1,:),min_con);
                x(1,:) = min(x(1,:),max_con);
                
                Ltmp = ones(1,Nseeds)*NaN;
                for ns = 1:Nseeds
                Ltmp(ns) = sim_fun(x(1,:),a0_d,a1_d,r_data,seed(nc,ns),a0_t);
                end
                % Prior:
                Pr = log(normpdf(x(2),theta_pm,theta_ps)) + log(normpdf(log(x(1)),k_pm,k_ps));
                L(1) = mean(Ltmp) + Pr;
                
                
                for i = 2:N % perform MCMC loops
                    
                    if mod(i,100) == 0
                        disp(strcat(['i = ',num2str(i)]))
                    end
                    
                   % Do delayed rejection
                   next_step = false;
                   step = 1; 
                   
                   while ~next_step
                   % Draw a candidate parameter set (one at a time)
                   x_cand = draw_cand(x(i-1,:),stream_null,CV,min_con,max_con,p,step);
                   
                   % Calculate likelihood
                   Ltmp = ones(1,Nseeds)*NaN;
                   for ns = 1:Nseeds
                   Ltmp(ns) = sim_fun(x_cand,a0_d,a1_d,r_data,seed(nc,ns),a0_t); 
                   end
                   L_cand = mean(Ltmp);
                   
                   % Prior:
                   Pr = log(normpdf(x_cand(2),theta_pm,theta_ps)) + log(normpdf(x_cand(1),k_pm,k_ps));
                   L_cand = L_cand + Pr;
                   
                   % Metropolis step
                   if ~isnan(L_cand)
                   ratio = exp(L_cand - L(i-1));
                   else
                       ratio = 0;
                   end
                   M = min(1,ratio) > rand(stream_null,1);
                   
                   % Apply delayed rejection
                   if M
                      x(i,:) = x_cand;
                      L(i,:) = L_cand;
                      next_step = true;
                      step = 1;
                   elseif ~M && step < stepmax % move to alternate proposal distribution
                       step = step+1;
                   else % if ~M and step >= stepmax  - actually reject
                       x(i,:) = x(i-1,:);
                       L(i,:) = L(i-1,:);
                       next_step = true;
                   end
                   
                   end % end delayed rejection while loop
                   
                   p = p+1;
                   if p > size(x,2)
                       p = 1;
                   end
                   
                   if ~isreal(L(i))
                       keyboard
                   end
                   
                end % end MCMC
                
                x_mc = nanmean(x);
                L_mc = nanmean(L);
                  
                LarvMort_st(1).(sp).chain(nc).k_mc = x_mc(1);
                LarvMort_st(1).(sp).chain(nc).theta_mc = x_mc(2:end);
                LarvMort_st(1).(sp).chain(nc).loglik = L_mc;
                   
                LarvMort_st(1).(sp).chain(nc).x_dist = x;
                LarvMort_st(1).(sp).chain(nc).L_dist = L;
                
                end % end multiple chains
                savename = strcat(['LarvMort_st_MCMC_',Sp_name,'_',St_name,'_',Datename,'.mat']);
                save(savename,'LarvMort_st')
                 
end % end for sp
    
               
% Accessory function     
function x_cand = draw_cand(x,stream_null,CV,min_con,max_con,p,step)
% Draw candidate parameter, randomly vary it, apply constraint

                   x_cand = x; 
                   
                   x_cand(p) = randn(stream_null,1)*CV(step,p).*x(p) + x(p);
                   
                   while x_cand(p) <= min_con(p) || x_cand(p) >= max_con(p)
                   
                   if x_cand(p) <= min_con(p)
                       x_cand(p) = min_con(p) + rand(stream_null,1)*( min_con(p) - x_cand(p) ); % reflect illegal values
                   end
                   
                   if x_cand(p) >= max_con(p)
                       x_cand(p) = max_con(p) - rand(stream_null,1)*( x_cand(p) - max_con(p) ); % reflect illegal values
                   end
                   
                   end

                
        
        
    
