function LLL = sim_fun(x,a1,a2,data,seed,init)

% Function to simulate the log-likelihood of the ratio of two negative
% binomial distributions
% For use with plankton_mort.m

% Author: Will White (whitejw@uncw.edu)
% This code accompanies:
% White JW, Morgan SG, Fisher JL. Improved estimates of
% planktonic larval mortality rates are lower than expected.
% Ecology
% Please cite accordingly.

% version: November 2013


K = x(1);  % clumping parameter
T = 10.^(x(2:end)); % mortality rate (input argument is on a log scale)

if length(T) == 1 % if estimating a single, joint mortality rate
T = repmat(T,[length(a1)+1,1]);
end


% Results are sensitive to the initial # of individuals used to
% seed.  So use the # observed in the dataset
N0 = mean(init);

% Set random stream so that a particular MCMC chain uses a consistent
% stream for all likelihood calculations
stream = RandStream('mt19937ar','seed',seed);
RandStream.setGlobalStream(stream);


for i = 1:(length(T)-1) % for each pair of stages

Theta_dist1 = repmat(T(i),1e5,1);
Theta_dist2 = repmat(T(i+1),1e5,1);

r1 = N0.*(1-exp(-Theta_dist1*a1(i)))./Theta_dist1;
r2 = N0.*exp(-Theta_dist1*a1(i)).*(1 - exp(-Theta_dist2*a2(i)) )./Theta_dist2;

% use r as estimate of mean of distribution
% must translate into nbinom parameter p
p1 = K./(r1 + K);
p2 = K./(r2 + K);

d1 = nbinrnd(K,p1);
d2 = nbinrnd(K,p2);

r = d1./d2; % ratio of distributions

% Keep track of 'unusual' values of the distribution
r0 = r== 0;
rnan = isnan(r);
rinf = isinf(r);
rOK = ~r0 & ~rnan & ~rinf;
r_n = length(r);

r = log10(r(rOK));  % use a log transform to spread the data out better
        
% approximate a pdf    
xi = -4:0.01:4;
f0 = histc(r,xi);

if any(f0>0)

f0(f0==0) = min(f0(f0>0)); % set a minimum value to avoid log of zero

f0 = f0./trapz(f0); % standardize by area (using trapz to estimate area under curve)

    if length(unique(r))>1 % if there is > 1 unique value in r

    % obtain pdf using histogram generated earlier
    dat = data{i};
    
    dat0 = dat == 0;
    datnan = isnan(dat);
    datinf = isinf(dat);
    datOK = ~dat0 & ~datnan & ~datinf;
    
    dat = log10(dat(datOK));
    
    [~, bin] = histc(dat,xi);
    bin(bin == 0) = length(xi);
    L = log( max(realmin, f0(bin) ));
    
    if flag
    keyboard
    end

   L(end+1) = log(max(realmin, mnpdf([sum(dat0), sum(datnan), sum(datinf), sum(datOK)],[sum(r0),sum(rnan),sum(rinf),sum(rOK)]./r_n)));
    
    else % if all the values of r are identical

    L = data{i}; % L is the same size as the dataset
    L(:) = realmin; % but all values are very small

        if any(data{i}==unique(r)) % if all the values of r are identical but equal to the actual data
    
         % those values get a very high likelihood
            L(data{i}==unique(r))=0.9;

        end % end if all values of r are identical

    L = log(L); % log-likelihood

    end


L = L(~isnan(L));
if isempty(L)
    L = NaN;
else
L = sum(L);
end

LL(i) = L;

else  % if all the data are NaNs
    
    LL(i) = NaN;
end % end if some data are non NaN
 
end % end loop over stages

if all(isnan(LL))
    LLL = NaN;
else
LLL = nansum(LL);
end
